@@ -2,18 +2,19 @@ setupLambdaCox <- function(X, y, Delta, alpha, lambda.min, nlambda, penalty.fact
   n <- nrow(X)
   p <- ncol(X)
 
-  ## Determine lambda.max
+  # Fit to unpenalized covariates
   ind <- which(penalty.factor!=0)
   if (length(ind)!=p) {
-    nullFit <- .Call("cdfit_cox_dh", X[, -ind, drop=FALSE], y, Delta, "lasso", 0, 0.001, as.integer(100), 3, penalty.factor[-ind],
-                    alpha, as.integer(p), as.integer(TRUE), as.integer(FALSE))
-    eta <- as.numeric(X[, -ind, drop=FALSE] %*% nullFit[[1]])
+    nullFit <- survival::coxph(survival::Surv(y, Delta) ~ X[, -ind, drop=FALSE])
+    eta <- nullFit$linear.predictors
     rsk <- rev(cumsum(rev(exp(eta))))
     s <- Delta - exp(eta)*cumsum(Delta/rsk)
   } else {
     w <- 1/(n-(1:n)+1)
     s <- Delta - cumsum(Delta*w)
   }
+
+  # Determine lambda.max
   zmax <- .Call("maxprod", X, s, ind, penalty.factor) / n
   lambda.max <- zmax/alpha
 
